#include "common.h"
#include "Archiver7ZIP.h"
#include "../util.h"
#include "../ConfigCode/config.h"
#include "../resource.h"
#include "../Components/TemporaryDirMgr.h"

const struct tagZIPVOLUMEUNIT{
	LPCTSTR DispName;
	LPCTSTR ParamName;
}ZIP_VOLUME_UNIT[]={
	{_T("MB"),_T("m")},
	{_T("KB"),_T("k")},
	{_T("GB"),_T("g")},
	{_T("Byte"),_T("b")}
};
const int ZIP_VOLUME_UNIT_MAX_NUM=sizeof(ZIP_VOLUME_UNIT)/sizeof(tagZIPVOLUMEUNIT);

class C7Zip32VolumeSizeDialog : public CDialogImpl<C7Zip32VolumeSizeDialog>,public CWinDataExchange<C7Zip32VolumeSizeDialog>
{
protected:
public:
	int VolumeSize;
	int SelectIndex;
	CComboBox Combo_VolumeUnit;
	enum {IDD = IDD_DIALOG_7Z_VOLUME_SIZE};

	// DDX}bv
	BEGIN_DDX_MAP(C7Zip32VolumeSizeDialog)
		DDX_INT(IDC_EDIT_7Z_VOLUME_SIZE, VolumeSize)
	END_DDX_MAP()

	// bZ[W}bv
	BEGIN_MSG_MAP_EX(C7Zip32VolumeSizeDialog)
		MSG_WM_INITDIALOG(OnInitDialog)
		COMMAND_ID_HANDLER_EX(IDOK, OnOK)
		COMMAND_ID_HANDLER_EX(IDCANCEL, OnCancel)
	END_MSG_MAP()

	LRESULT OnInitDialog(HWND hWnd, LPARAM lParam){
		Combo_VolumeUnit=GetDlgItem(IDC_COMBO_7Z_VOLUME_UNIT);
		SelectIndex=0;
		for(int i=0;i<ZIP_VOLUME_UNIT_MAX_NUM;i++){
			Combo_VolumeUnit.InsertString(-1,ZIP_VOLUME_UNIT[i].DispName);
		}
		Combo_VolumeUnit.SetCurSel(0);
		return TRUE;
	}

	void OnOK(UINT uNotifyCode, int nID, HWND hWndCtl){
		if(!DoDataExchange(TRUE)){
			return;
		}
		if(VolumeSize<=0){
			::SetFocus(GetDlgItem(IDC_EDIT_7Z_VOLUME_SIZE));
			::SendMessage(GetDlgItem(IDC_EDIT_7Z_VOLUME_SIZE),EM_SETSEL,0,-1);
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_VALUE_MUST_BE_PLUS)));
			return;
		}
		SelectIndex=Combo_VolumeUnit.GetCurSel();
		if(SelectIndex<0||SelectIndex>=ZIP_VOLUME_UNIT_MAX_NUM)return;
		EndDialog(nID);
	}

	void OnCancel(UINT uNotifyCode, int nID, HWND hWndCtl){
		EndDialog(nID);
	}
};

//---------------

CArchiver7ZIP::CArchiver7ZIP():
ArchiverSetUnicodeMode(NULL),
ArchiverGetArchiveType(NULL)
{
//	TRACE(_T("CArchiver7ZIP()\n"));
//	Separator=_T('\\');
	m_nRequiredVersion=442;
	m_nRequiredSubVersion=4;
	m_strDllName=_T("7-ZIP32.DLL");
	m_AstrPrefix="SevenZip";
}

CArchiver7ZIP::~CArchiver7ZIP()
{
//	TRACE(_T("~CArchiver7ZIP()\n"));
	FreeDLL();
}

LOAD_RESULT CArchiver7ZIP::LoadDLL(CConfigManager &Config,bool bSilent)
{
	FreeDLL();

	//NX̃\bhĂ
	LOAD_RESULT res=CArchiverDLL::LoadDLL(Config,bSilent);
	if(LOAD_RESULT_OK!=res){
		return res;
	}

	//UNICODE[hݒp
	ArchiverSetUnicodeMode=(COMMON_ARCHIVER_SETUNICODEMODE)GetProcAddress(m_hInstDLL,"SevenZipSetUnicodeMode");
	if(NULL==ArchiverSetUnicodeMode){
		if(!bSilent){
			CString msg;
			msg.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("SevenZipSetUnicodeMode"));
			ErrorMessage(msg);
		}
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}
	//UNICODE[hɐݒ
	if(!ArchiverSetUnicodeMode(TRUE)){
		//UNICODEɂłȂ玸sƂ
		return LOAD_RESULT_INVALID;
	}

	//A[JCuʔp
	ArchiverGetArchiveType=(COMMON_ARCHIVER_GETARCHIVETYPE)GetProcAddress(m_hInstDLL,"SevenZipGetArchiveType");
	if(NULL==ArchiverGetArchiveType){
		if(!bSilent){
			CString msg;
			msg.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("SevenZipGetArchiveType"));
			ErrorMessage(msg);
		}
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}
	return LOAD_RESULT_OK;
}

void CArchiver7ZIP::FreeDLL()
{
	if(m_hInstDLL){
		ArchiverSetUnicodeMode=NULL;
		ArchiverGetArchiveType=NULL;
		CArchiverDLL::FreeDLL();
	}
}

bool CArchiver7ZIP::Compress(LPCTSTR ArcFileName,std::list<CString> &ParamList,CConfigManager &Config,const PARAMETER_TYPE Type,int Options,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	ASSERT(0!=_tcslen(ArcFileName));
	TRACE(_T("ArcFileName=%s\n"),ArcFileName);

	//ɂ̏ꍇ͎ȉ𓀂͏oȂ
	if((Options&COMPRESS_SPLIT)&&(Options&COMPRESS_SFX)){
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_CANNOT_SPLIT_SFX)));
		Options&=~COMPRESS_SFX;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("zip"))){
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE)));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//============================================
	// ȉ𓀃t@Cpe|t@C擾
	//============================================
	TCHAR SFXTemporaryFileName[_MAX_PATH+1];
	FILL_ZERO(SFXTemporaryFileName);
	bool bZIPSFX=((0!=(Options&COMPRESS_SFX))&&(PARAMETER_ZIP==Type));
	if(bZIPSFX){
		//2iK쐬
		if(!UtilGetTemporaryFileName(SFXTemporaryFileName,_T("sfx"))){
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE)));
			return false;
		}
		ASSERT(0!=_tcslen(SFXTemporaryFileName));
		DeleteFile(SFXTemporaryFileName);//S~t@C
	}

	//ȉ𓀗pW[pX
	TCHAR SFXModulePath[_MAX_PATH+1];

	//====================================================
	// X|Xt@CɈkΏۃt@CL
	// A[JCut@C̓R}hCŒڎw肷
	//====================================================
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile)
		{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			return false;
		}

		TRACE(_T("X|Xt@Cւ̏\n"));
		std::list<CString>::iterator ite;
		for(ite=ParamList.begin();ite!=ParamList.end();ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	switch(Type){
	case PARAMETER_ZIP:	//ZIP`ňk
		Config.LoadConfig(CONFIG_ZIP);
		Param+=
			_T("a ")			//k
			_T("-tzip ")		//ZIP`ňk
			_T("-r0 ")			//TufBNg
		;
		switch(Config.ZIP.CompressType){	//k
		case ZIP_COMPRESS_DEFLATE:
			Param+=_T("-mm=Deflate ");
			break;
		case ZIP_COMPRESS_DEFLATE64:
			Param+=_T("-mm=Deflate64 ");
			break;
		case ZIP_COMPRESS_BZIP2:
			Param+=_T("-mm=BZip2 ");
			break;
		case ZIP_COMPRESS_COPY:
			Param+=_T("-mm=Copy ");
			break;
		}
		switch(Config.ZIP.CompressLevel){	//kx
		case ZIP_COMPRESS_LEVEL0:
			Param+=_T("-mx=0 ");
			break;
		case ZIP_COMPRESS_LEVEL5:
			Param+=_T("-mx=5 ");
			break;
		case ZIP_COMPRESS_LEVEL9:
			Param+=_T("-mx=9 ");
			break;
		}
		if((ZIP_COMPRESS_DEFLATE==Config.ZIP.CompressType)||(ZIP_COMPRESS_DEFLATE64==Config.ZIP.CompressType)){
			if(Config.ZIP.SpecifyDeflateMemorySize){
				CString temp;
				temp.Format(_T("-mfb=%d "),Config.ZIP.DeflateMemorySize);
				Param+=temp;
			}
			if(Config.ZIP.SpecifyDeflatePassNumber){
				CString temp;
				temp.Format(_T("-mpass=%d "),Config.ZIP.DeflatePassNumber);
				Param+=temp;
			}
		}
		if(Options&COMPRESS_PASSWORD){	//pX[ht
			Param+=_T("-p ");
		}
		if(bZIPSFX){
			//BZip2̎ȉ𓀂̓T|[gĂȂ
			if(ZIP_COMPRESS_BZIP2==Config.ZIP.CompressType){
				ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_ZIP_SFX_DONT_SUPPORT_BZIP2)));
				DeleteFile(ResponceFileName);
				return false;
			}
			FILL_ZERO(SFXModulePath);
			LPTSTR lptemp;
			if(!SearchPath(NULL,_T("SFX32GUI.DAT"),NULL,_MAX_PATH,SFXModulePath,&lptemp)){
//			if(!UtilGetSFXModuleFullPath(SFXModulePath,_T("SFX32GUI.DAT"))){
				CString msg;
				msg.Format(IDS_ERROR_SFX_MODULE_NOT_FOUND,_T("SFX32GUI.DAT"));
				DeleteFile(ResponceFileName);
				ErrorMessage(msg);
				return false;
			}
		}
		break;
	case PARAMETER_7Z:	//7z`ňk
		Config.LoadConfig(CONFIG_7Z);
		Param+=
			_T("a ")			//k
			_T("-t7z ")			//7-Zip`ňk
			_T("-r0 ")			//TufBNg
		;
		if(Config.SEVEN_ZIP.UsePreset){
			//vZbgk[h
			switch(Config.SEVEN_ZIP.CompressLevel){	//kx
			case SEVEN_ZIP_COMPRESS_LEVEL0:
				Param+=_T("-mx=0 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL1:
				Param+=_T("-mx=1 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL5:
				Param+=_T("-mx=5 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL7:
				Param+=_T("-mx=7 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL9:
				Param+=_T("-mx=9 ");
				break;
			}
		}else{
			switch(Config.SEVEN_ZIP.CompressType){	//k
			case SEVEN_ZIP_COMPRESS_LZMA:
				Param+=_T("-m0=LZMA:a=");
				switch(Config.SEVEN_ZIP.LZMA_Mode){	//LZMAk[h
				case SEVEN_ZIP_LZMA_MODE0:
					Param+=_T("0 ");
					break;
				case SEVEN_ZIP_LZMA_MODE1:
					Param+=_T("1 ");
					break;
				case SEVEN_ZIP_LZMA_MODE2:
					Param+=_T("2 ");
					break;
				}
				break;
			case SEVEN_ZIP_COMPRESS_PPMD:
				Param+=_T("-m0=PPMd ");
				break;
			case SEVEN_ZIP_COMPRESS_BZIP2:
				Param+=_T("-m0=BZip2 ");
				break;
			case SEVEN_ZIP_COMPRESS_DEFLATE:
				Param+=_T("-m0=Deflate ");
				break;
			case SEVEN_ZIP_COMPRESS_COPY:
				Param+=_T("-m0=Copy ");
				break;
			}
		}
		if(Config.SEVEN_ZIP.SolidMode){	//\bh[h
			Param+=_T("-ms=on ");
		}else{
			Param+=_T("-ms=off ");
		}
		//------------
		// wb_k
		//------------
		if(Config.SEVEN_ZIP.HeaderCompression){	//wb_k
			Param+=_T("-mhc=on ");
			if(Config.SEVEN_ZIP.FullHeaderCompression){	//wb_Sk
				Param+=_T("-mhcf=on ");
			}else{
				Param+=_T("-mhcf=off ");
			}
		}else{
			Param+=_T("-mhc=off ");
			Param+=_T("-mhcf=off ");
		}

		if(Options&COMPRESS_PASSWORD){	//pX[ht
			Param+=_T("-p ");
			if(Config.SEVEN_ZIP.HeaderEncryption){	//wb_Í
				Param+=_T("-mhe=on ");
			}
			else{
				Param+=_T("-mhe=off ");
			}
		}
		if(Options&COMPRESS_SFX){
			//BZip2̎ȉ𓀂̓T|[gĂȂ
			if(SEVEN_ZIP_COMPRESS_BZIP2==Config.SEVEN_ZIP.CompressType){
				ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_7Z_SFX_DONT_SUPPORT_BZIP2)));
				DeleteFile(ResponceFileName);
				return false;
			}
			Param+=_T("-sfx ");
		}
		break;
	}
	//
	if(Options&COMPRESS_SPLIT){
		C7Zip32VolumeSizeDialog vsd;
		if(IDOK!=vsd.DoModal())return false;
		CString temp;
		temp.Format(_T("-v%d%s "),vsd.VolumeSize,ZIP_VOLUME_UNIT[vsd.SelectIndex].ParamName);
		Param+=temp;
	}

	Param+=_T("-scsUTF-8 ");	//X|Xt@C̃R[hy[Ww

	//kt@Cw
	if(bZIPSFX){
		Param+=_T("\"");
		Param+=SFXTemporaryFileName;
		Param+=_T("\" ");
	}
	else{
		Param+=_T("\"");
		Param+=ArcFileName;
		Param+=_T("\" ");
	}

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;

	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	//G[Oo
	if(!bZIPSFX||0!=Ret){
		if(bZIPSFX){
			DeleteFile(SFXTemporaryFileName);
		}
		return 0==Ret;
	}

	//==================================
	// ȉ𓀏ɂɕϊ(oCi)
	//==================================
	//o͐t@CJ
	HANDLE hArcFile=CreateFile(ArcFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(INVALID_HANDLE_VALUE==hArcFile){
		CString msg;
		msg.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
		ErrorMessage(msg);
		DeleteFile(ArcFileName);
		DeleteFile(SFXTemporaryFileName);
		return false;
	}
	{
		//SFXW[ǂݎ胂[hŊJ
		HANDLE hSFXModuleFile=CreateFile(SFXModulePath,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hSFXModuleFile)
		{
			CString msg;
			msg.Format(IDS_ERROR_SFX_MODULE_CANNOT_ACCESS,SFXModulePath);
			ErrorMessage(msg);
			CloseHandle(hArcFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		//SFXW[̒gRs[
		int Result=UtilAppendFile(hArcFile,hSFXModuleFile);
		if(Result>0){	//ǂݎG[
			CString msg;
			msg.Format(IDS_ERROR_SFX_MODULE_CANNOT_ACCESS,SFXModulePath);
			ErrorMessage(msg);
			CloseHandle(hArcFile);
			CloseHandle(hSFXModuleFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		else if(Result<0){	//݃G[
			CString msg;
			msg.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
			ErrorMessage(msg);
			CloseHandle(hArcFile);
			CloseHandle(hSFXModuleFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		CloseHandle(hSFXModuleFile);
	}
	{
		//e|t@Cǂݎ胂[hŊJ
		HANDLE hSFXTempFile=CreateFile(SFXTemporaryFileName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hSFXTempFile)
		{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			CloseHandle(hArcFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		//e|t@C̒gRs[
		int Result=UtilAppendFile(hArcFile,hSFXTempFile);
		if(Result>0){	//ǂݎG[
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			CloseHandle(hArcFile);
			CloseHandle(hSFXTempFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		else if(Result<0){	//݃G[
			CString msg;
			msg.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
			ErrorMessage(msg);
			CloseHandle(hArcFile);
			CloseHandle(hSFXTempFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		CloseHandle(hSFXTempFile);
		DeleteFile(SFXTemporaryFileName);
	}
	CloseHandle(hArcFile);

	return true;
}

bool CArchiver7ZIP::Extract(LPCTSTR ArcFileName,CConfigManager &Config,bool bSafeArchive,LPCTSTR OutputDir,CString &strLog)
{
	if(!IsOK()){
		return false;
	}
	if(!bSafeArchive){
		//CString msg;
		//msg.Format(IDS_ERROR_DANGEROUS_ARCHIVE,ArcFileName);
		//ErrorMessage(msg);
		strLog.Format(IDS_ERROR_DANGEROUS_ARCHIVE,ArcFileName);
		return false;
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	//𓀃p[^
	Param+=_T("x ");			//
	if(Config.Common.Extract.ForceOverwrite){
		//㏑
		Param+=_T("-aoa ");
	}

	Param+=_T("-scsUTF-8 ");	//X|Xt@C̃R[hy[Ww

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//o͐w
	Param+=_T("-o\"");
	Param+=OutputDir;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;

	return 0==Ret;
}

//X|Xt@CɃt@CGXP[vsŏށB
//Lȃt@CnhNULLłȂt@CnƁB
void CArchiver7ZIP::WriteResponceFile(HANDLE hFile,LPCTSTR fname)
{
	CPath strPath(fname);

	if(strPath.IsDirectory()){
		strPath.UnquoteSpaces();
		strPath.Append(_T("*"));
	}
	strPath.QuoteSpaces();

	DWORD dwWritten=0;
	//t@C+so
	std::vector<BYTE> cArray;
	UtilToUTF8(cArray,strPath+_T("\r\n"));
	WriteFile(hFile,&cArray[0],(cArray.size()-1)*sizeof(BYTE),&dwWritten,NULL);

/*	//t@Co
	CStringA strBuffer(strPath);
	WriteFile(hFile,(LPCSTR)strBuffer,strBuffer.GetLength()*sizeof(char),&dwWritten,NULL);

	//t@C؂邽߂̉s
	char CRLF[]="\r\n";
	WriteFile(hFile,&CRLF,strlen(CRLF)*sizeof(char),&dwWritten,NULL);*/
}


//=============================================================
// SevenZipGetFileName()̏o͌ʂɁAi[ꂽt@C
// pXĂ邩ǂʂAdtH_쐬h
//=============================================================
bool CArchiver7ZIP::ExamineArchive(LPCTSTR ArcFileName,bool bSkipDir,bool &bInFolder,bool &bSafeArchive,CString &BaseDir)
{
	//UNICODESǂ肷
	if(!IsOK()){
		return false;
	}

	if(!InspectArchiveBegin(ArcFileName)){
		CString msg;
		msg.Format(IDS_ERROR_OPEN_ARCHIVE,ArcFileName);
		ErrorMessage(msg);
		return false;
	}

	bInFolder=true;
	bool bSureDir=false;	//BaseDirɓĂ镶񂪊mɃtH_łȂtrue
	TRACE(_T("========\n"));

	while(InspectArchiveNext()){
		CString Buffer;
		InspectArchiveGetFileName(Buffer);
		Buffer.Replace(_T('\\'),_T('/'));		//pX؂蕶̒u
		TRACE(_T("%s\n"),Buffer);

		/*
		Separator('/' or '\')͊i[t@C̐擪ɂ܂܂ĂĂׂł̂ŁA
		i[t@C̐擪ɂSeparatorĂtH_Ɋi[ꂽԂƂ͌ȂȂB
		SeparatorMaxRepeat葽ƕsƂ
		AMaxRepeat-1̂Ƃ̓G[Ƃ͂Ȃ
		*/
		const int Length=Buffer.GetLength();
		int StartIndex=0;
		for(;StartIndex<Length;StartIndex++){
			//擪'/'Ƃ΂Ă
#if defined(_UNICODE)||defined(UNICODE)
			if(_T('/')!=Buffer[StartIndex])break;
#else
			if(_MBC_SINGLE==_mbsbtype((const unsigned char *)(LPCTSTR)Buffer,StartIndex)){
				if(_T('/')!=Buffer[StartIndex])break;
			}
			else{	//SpȂ'/'ł͂Ȃ
				break;
			}
#endif//defined(_UNICODE)||defined(UNICODE)
		}
		if(!UtilIsSafeUnicode((LPCTSTR)Buffer+StartIndex)){	//댯UNICODEw肪΁A댯ȃt@CƌȂ
			//čI
			InspectArchiveEnd();
			bSafeArchive=false;
			bInFolder=false;
			return true;
		}

		//͓dfBNg
		//łɓdfBNg肪tĂꍇ͈Ŝ݂ɓO

		int FoundIndex=0;
		while(bInFolder){
			FoundIndex=Buffer.Find(_T('/'),StartIndex);
			if(-1==FoundIndex){	//'/'i[t@C̐擪ȊOɊ܂܂Ȃꍇ
				if(!BaseDir.IsEmpty()&&BaseDir==Buffer){
					bSureDir=true;	//BaseDirtH_łƊmFꂽ
					break;
				}
				else if(BaseDir.IsEmpty()){
					//tH_̌'/'tȂA[JCo
					//̂ŏɏoĂƂ́AtH_Ɖ肷
					BaseDir=Buffer;
					bSureDir=false;
					break;
				}
			}
			CString Dir=Buffer.Mid(StartIndex,FoundIndex-StartIndex);	//SeparatoȓO܂ł̕(fBNgɑ)𔲂oĂ
			//܂łׂ̒DirEmptył͂ȂƂۏ؂Ă
			//܂A댯ł͂ȂƂĂ
			TRACE(_T("Base=%s,Dir=%s\n"),BaseDir,Dir);

			if(_T('.')==Dir){	//./΃fBNgwƂĂ͖
				StartIndex=FoundIndex+1;
				continue;
			}
			if(BaseDir.IsEmpty()){
				BaseDir=Dir;
				bSureDir=true;
			}
			else if(BaseDir!=Dir){
				bInFolder=false;
			}
			else bSureDir=true;	//BaseDirfBNgƊmFꂽ
			break;
		}
	}
	TRACE(_T("========\n"));

	InspectArchiveEnd();
	bSafeArchive=true;

	//tH_ɓĂ悤ł͂邪AfBNgƉ肳ꂽ̏ꍇ
	if(bInFolder&&!bSureDir)bInFolder=false;
	return true;
}

bool CArchiver7ZIP::ExtractSpecifiedOnly(LPCTSTR ArcFileName,LPCTSTR OutputDir,std::list<CString> &FileList,CString &strLog,bool bUsePath)
{
	if(!IsOK()){
		return false;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("7zp"))){
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE)));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//𓀑Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile)
		{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			return false;
		}

		std::list<CString>::iterator ite=FileList.begin();
		const std::list<CString>::iterator end=FileList.end();
		for(;ite!=end;ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//𓀃p[^
	if(bUsePath){
		Param+=_T("x ");	//pXt
	}else{
		Param+=_T("e ");		//fBNgȂŉ
	}
	Param+=
		_T("-r- ")		//ċA
		_T("-scsUTF-8 ")	//X|Xt@C̃R[hy[Ww
	;

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//o͐w
	Param+=_T("-o\"");
	Param+=OutputDir;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	TRACE(_T("ArchiveHandlerĂяo\nCommandline Parameter:%s\n"),Param);
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}


bool CArchiver7ZIP::DeleteFileFromArchive(LPCTSTR ArcFileName,std::list<CString>& FileList,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("7zp"))){
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE)));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//폜Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile)
		{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			return false;
		}

		std::list<CString>::iterator ite=FileList.begin();
		const std::list<CString>::iterator end=FileList.end();
		for(;ite!=end;ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//폜p[^
	Param+=
		_T("d ")	//폜
		_T("-r- ")	//ċA
		_T("-scsUTF-8 ")	//X|Xt@C̃R[hy[Ww
	;

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");


	TRACE(_T("ArchiveHandlerĂяo\nCommandline Parameter:%s\n"),Param);
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}

ARCRESULT CArchiver7ZIP::TestArchive(LPCTSTR ArcFileName,CString &strLog)
{
	TRACE(_T("TestArchive() called.\n"));
	ASSERT(IsOK());
	if(!IsOK()){
		return TEST_ERROR;
	}

	//tR}hɂeXgĂ
	CString Param=
		_T("t ")			//eXg
		_T("\"")
	;
	Param+=ArcFileName;
	Param+=_T("\"");

	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;

/*	//UNICODẼ`FbN
	bool bInFolder;
	CString basedir;
	bool bSafeArchive=false;
	bool bRet=ExamineArchive(ArcFileName,false,bInFolder,bSafeArchive,basedir);
	if(!bRet){	//Ɏs
		strLog+=_T("\n\n");
		strLog+=CString(MAKEINTRESOURCE(IDS_ERROR_EXAMINEARCHIVE));
		Ret=-1;
	}else if(!bSafeArchive){	//UNICODE䕶o
		strLog+=_T("\n\n");
		strLog+=CString(MAKEINTRESOURCE(IDS_ERROR_UNICODECONTROL));
		Ret=-1;
	}*/


	if(Ret==0)return TEST_OK;
	else return TEST_NG;
}


//-------------------------------
//---UNICODEłI[o[Ch---
//-------------------------------
BOOL CArchiver7ZIP::CheckArchive(LPCTSTR _szFileName)
{
	if(!ArchiverCheckArchive){
		ASSERT(ArchiverCheckArchive);
		return false;
	}
	return ArchiverCheckArchive(C2UTF8(_szFileName),1);	//CHECKARCHIVE_BASIC
}

int CArchiver7ZIP::GetFileCount(LPCTSTR _szFileName)
{
	if(!ArchiverGetFileCount){
		return -1;
	}
	return ArchiverGetFileCount(C2UTF8(_szFileName));
}


bool CArchiver7ZIP::InspectArchiveBegin(LPCTSTR ArcFileName)
{
	TRACE(_T("CArchiverDLL::InspectArchiveBegin()\n"));
	ASSERT(ArchiverOpenArchive);
	if(!ArchiverOpenArchive){
		return false;
	}
	if(m_hInspectArchive){
		ASSERT(!"Close the Archive First!!!\n");
		return false;
	}
	m_hInspectArchive=ArchiverOpenArchive(NULL,C2UTF8(ArcFileName),m_dwInspectMode);
	if(!m_hInspectArchive){
		TRACE(_T("Failed to Open Archive\n"));
		return false;
	}
	m_bInspectFirstTime=true;

	FILL_ZERO(m_IndividualInfo);
	return true;
}

bool CArchiver7ZIP::InspectArchiveGetFileName(CString &FileName)
{
	if(ArchiverGetFileName){
		if(!m_hInspectArchive){
			ASSERT(!"Open an Archive First!!!\n");
			return false;
		}
		std::vector<BYTE> szBuffer(FNAME_MAX32*2+1);
		ArchiverGetFileName(m_hInspectArchive,(LPCSTR)&szBuffer[0],FNAME_MAX32*2);
		UtilToUNICODE(FileName,&szBuffer[0],szBuffer.size(),UTILCP_UTF8);
		return true;
	}
	else{
		ASSERT(LOAD_DLL_STANDARD!=m_LoadLevel);
		if(!m_hInspectArchive){
			ASSERT(!"Open an Archive First!!!\n");
			return false;
		}
		UtilToUNICODE(FileName,(LPCBYTE)m_IndividualInfo.szFileName,sizeof(m_IndividualInfo.szFileName),UTILCP_UTF8);
		return true;
	}
}

bool CArchiver7ZIP::InspectArchiveGetMethodString(CString &strMethod)
{
	if(!m_hInspectArchive){
		ASSERT(!"Open an Archive First!!!\n");
		return false;
	}

	//\0ŏIĂ邩ǂۏ؂łȂ
	char szBuffer[32]={0};

	if(ArchiverGetMethod){
		if(0!=ArchiverGetMethod(m_hInspectArchive,szBuffer,31)){
			//\̂擾
			//strncpy(szBuffer,m_IndividualInfo.szMode,8);
			strncpy_s(szBuffer,m_IndividualInfo.szMode,8);
		}
	}
	else{
		//\̂擾
		//strncpy(szBuffer,m_IndividualInfo.szMode,8);
		strncpy_s(szBuffer,m_IndividualInfo.szMode,8);
	}

	//i[
	UtilToUNICODE(strMethod,(LPCBYTE)szBuffer,9,UTILCP_UTF8);
	return true;
}

bool CArchiver7ZIP::AddFileToArchive(LPCTSTR ArcFileName,std::list<CString> &FileList,CConfigManager &Config,LPCTSTR lpDestDir,CString &strLog)
{
	// X|Xt@Cpe|t@C擾
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("zip"))){
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE)));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//===ꎞIɃt@CRs[
	//---\ŏI_pX擾
	CPath strBasePath;
	UtilGetBaseDirectory(strBasePath,FileList);
	TRACE(_T("%s\n"),strBasePath);

	//---e|ɑΏۃt@CRs[
	//e|
	CTemporaryDirectoryManager tdm(_T("lhaf"));
	CPath strDestPath(tdm.GetDirPath());
	strDestPath+=lpDestDir;
	UtilMakeSureDirectoryPathExists(strDestPath);

	// kΏۃt@CC
	const int BasePathLength=((CString)strBasePath).GetLength();
	CString strSrcFiles;	//Rs[t@C̈ꗗ
	CString strDestFiles;	//Rs[t@C̈ꗗ
	std::list<CString>::iterator ite;
	for(ite=FileList.begin();ite!=FileList.end();++ite){
		//x[XpXɑ΃pX擾 : ʂłpX̕Jbg
		LPCTSTR lpSrc((LPCTSTR)(*ite)+BasePathLength);

		//葤t@Cw
		strSrcFiles+=(strBasePath+lpSrc);	//PathAppend
		strSrcFiles+=_T('|');
		//󂯑t@Cw
		strDestFiles+=strDestPath+lpSrc;
		strDestFiles+=_T('|');
	}
	strSrcFiles+=_T('|');
	strDestFiles+=_T('|');

	//'|''\0'ɕϊ
	std::vector<TCHAR> srcBuf(strSrcFiles.GetLength()+1);
	UtilMakeFilterString(strSrcFiles,&srcBuf[0],srcBuf.size());
	std::vector<TCHAR> destBuf(strDestFiles.GetLength()+1);
	UtilMakeFilterString(strDestFiles,&destBuf[0],destBuf.size());

	//t@Ce
	SHFILEOPSTRUCT fileOp={0};
	fileOp.wFunc=FO_COPY;
	fileOp.fFlags=FOF_MULTIDESTFILES|FOF_NOCONFIRMATION|FOF_NOCONFIRMMKDIR|FOF_NOCOPYSECURITYATTRIBS|FOF_NO_CONNECTED_ELEMENTS;
	fileOp.pFrom=&srcBuf[0];
	fileOp.pTo=&destBuf[0];

	//Rs[s
	if(::SHFileOperation(&fileOp)){
		//G[
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_FILE_COPY));
		return false;
	}else if(fileOp.fAnyOperationsAborted){
		//LZ
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_USERCANCEL));
		return false;
	}

	//JgfBNgݒ
	::SetCurrentDirectory(tdm.GetDirPath());
	// ɁAX|Xt@CɃA[JCuшkΏۃt@CL
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			return false;
		}
		//X|Xt@Cւ̏
		//SĈk
		WriteResponceFile(hFile,_T("*"));
		CloseHandle(hFile);
	}


	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	ASSERT(ArchiverGetArchiveType);
	switch(ArchiverGetArchiveType(C2UTF8(ArcFileName))){
	case 1:	//ZIP`ňk
		TRACE(_T("This is zip\n"));
		Config.LoadConfig(CONFIG_ZIP);
		Param+=
			_T("a ")			//k
			_T("-tzip ")		//ZIP`ňk
			_T("-r0 ")			//TufBNg
		;
		switch(Config.ZIP.CompressType){	//k
		case ZIP_COMPRESS_DEFLATE:
			Param+=_T("-mm=Deflate ");
			break;
		case ZIP_COMPRESS_DEFLATE64:
			Param+=_T("-mm=Deflate64 ");
			break;
		case ZIP_COMPRESS_BZIP2:
			Param+=_T("-mm=BZip2 ");
			break;
		case ZIP_COMPRESS_COPY:
			Param+=_T("-mm=Copy ");
			break;
		}
		switch(Config.ZIP.CompressLevel){	//kx
		case ZIP_COMPRESS_LEVEL0:
			Param+=_T("-mx=0 ");
			break;
		case ZIP_COMPRESS_LEVEL5:
			Param+=_T("-mx=5 ");
			break;
		case ZIP_COMPRESS_LEVEL9:
			Param+=_T("-mx=9 ");
			break;
		}
		if((ZIP_COMPRESS_DEFLATE==Config.ZIP.CompressType)||(ZIP_COMPRESS_DEFLATE64==Config.ZIP.CompressType)){
			if(Config.ZIP.SpecifyDeflateMemorySize){
				CString temp;
				temp.Format(_T("-mfb=%d "),Config.ZIP.DeflateMemorySize);
				Param+=temp;
			}
			if(Config.ZIP.SpecifyDeflatePassNumber){
				CString temp;
				temp.Format(_T("-mpass=%d "),Config.ZIP.DeflatePassNumber);
				Param+=temp;
			}
		}
		break;
	case 2:	//7z`ňk
		TRACE(_T("This is 7z\n"));
		Config.LoadConfig(CONFIG_7Z);
		Param+=
			_T("a ")			//k
			_T("-t7z ")			//7-Zip`ňk
			_T("-r0 ")			//TufBNg
		;
		if(Config.SEVEN_ZIP.UsePreset){
			//vZbgk[h
			switch(Config.SEVEN_ZIP.CompressLevel){	//kx
			case SEVEN_ZIP_COMPRESS_LEVEL0:
				Param+=_T("-mx=0 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL1:
				Param+=_T("-mx=1 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL5:
				Param+=_T("-mx=5 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL7:
				Param+=_T("-mx=7 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL9:
				Param+=_T("-mx=9 ");
				break;
			}
		}else{
			switch(Config.SEVEN_ZIP.CompressType){	//k
			case SEVEN_ZIP_COMPRESS_LZMA:
				Param+=_T("-m0=LZMA:a=");
				switch(Config.SEVEN_ZIP.LZMA_Mode){	//LZMAk[h
				case SEVEN_ZIP_LZMA_MODE0:
					Param+=_T("0 ");
					break;
				case SEVEN_ZIP_LZMA_MODE1:
					Param+=_T("1 ");
					break;
				case SEVEN_ZIP_LZMA_MODE2:
					Param+=_T("2 ");
					break;
				}
				break;
			case SEVEN_ZIP_COMPRESS_PPMD:
				Param+=_T("-m0=PPMd ");
				break;
			case SEVEN_ZIP_COMPRESS_BZIP2:
				Param+=_T("-m0=BZip2 ");
				break;
			case SEVEN_ZIP_COMPRESS_DEFLATE:
				Param+=_T("-m0=Deflate ");
				break;
			case SEVEN_ZIP_COMPRESS_COPY:
				Param+=_T("-m0=Copy ");
				break;
			}
		}
		if(Config.SEVEN_ZIP.SolidMode){	//\bh[h
			Param+=_T("-ms=on ");
		}else{
			Param+=_T("-ms=off ");
		}
		//------------
		// wb_k
		//------------
		if(Config.SEVEN_ZIP.HeaderCompression){	//wb_k
			Param+=_T("-mhc=on ");
			if(Config.SEVEN_ZIP.FullHeaderCompression){	//wb_Sk
				Param+=_T("-mhcf=on ");
			}else{
				Param+=_T("-mhcf=off ");
			}
		}else{
			Param+=_T("-mhc=off ");
			Param+=_T("-mhcf=off ");
		}
		break;
	default:
		ASSERT(!"This code cannot be run");
		//G[ʓ|Ȃ̂ŕĂB
		return false;
	}

	Param+=_T("-scsUTF-8 ");	//X|Xt@C̃R[hy[Ww

	//kt@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];

	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}
